
$identify("org/mathdox/formulaeditor/modules/set1/notin.js");

$require("org/mathdox/parsing/ParserGenerator.js");
$require("org/mathdox/formulaeditor/parsing/openmath/OpenMathParser.js");
$require("org/mathdox/formulaeditor/parsing/expression/ExpressionContextParser.js");
$require("org/mathdox/formulaeditor/semantics/MultaryOperation.js");
$require("org/mathdox/formulaeditor/semantics/Keyword.js");
$require("org/mathdox/formulaeditor/parsing/openmath/KeywordList.js");

$main(function(){

  var symbol = {
    onscreen : "∉",
    openmath : null, // use default with model:cd and model:name
    mathml   : "<mo>∉</mo>"
  };

  /**
   * Define a semantic tree node that represents set1.notin.
   */
  org.mathdox.formulaeditor.semantics.Set1Notin =
    $extend(org.mathdox.formulaeditor.semantics.MultaryOperation, {

      symbol : {

        onscreen : symbol.onscreen,
        openmath : "<OMS cd='set1' name='notin'/>",
        mathml   : symbol.mathml

      },

      associative : true,
      precedence : 110

    });
  
  /**
   * Extend the OpenMathParser object with parsing code for set1.notin.
   */
  org.mathdox.formulaeditor.parsing.openmath.OpenMathParser =
    $extend(org.mathdox.formulaeditor.parsing.openmath.OpenMathParser, {

    /**
     * Returns an equality object based on the OpenMath node.
     */
    handleSet1Notin : function(node) {

      // parse the children of the OMA
      var children = node.childNodes;
      var operands = [];
      for (var i=1; i<children.length; i++) {
        operands.push(this.handle(children.item(i)));
      }

      // construct the corresponding object
      var result = new org.mathdox.formulaeditor.semantics.Set1Notin();
      result.initialize.apply(result, operands);

      return result;
    }

  });

  org.mathdox.formulaeditor.parsing.openmath.KeywordList["set1__notin"] = new org.mathdox.formulaeditor.semantics.Keyword("set1", "notin", symbol, "infix");

  /**
   * Add the parsing code for an infix-once symbol.
   */
  var semantics = org.mathdox.formulaeditor.semantics;
  var pG = new org.mathdox.parsing.ParserGenerator();

  
  
    // only one expression, same on screen
    org.mathdox.formulaeditor.parsing.expression.ExpressionContextParser.addFunction( 
      function(context) { 
        
        return {
          // expression110 = notin | super.expression110
          expression110 : function() {
            var parent = arguments.callee.parent;
            pG.alternation(
              pG.rule("set1notin"),
              parent.expression110).apply(this, arguments);
          },
    
          // set1notin = 
          //    expression110 "∉" expression120
          set1notin : function() {
            var parent = arguments.callee.parent;
            return pG.transform(
              pG.concatenation(
                pG.rule("expression110"),
                pG.literal("∉"),
                pG.rule("expression120")
              ),
              function(result) {
                return parent.infix_Update(new semantics.Set1Notin(result[0], result[2]));
              }
            ).apply(this, arguments);
          }
        };
        
      }
    );
  
  
});
